
local rematch = Rematch
local settings

-- the scrollable options
-- when adding options:
-- 1. make the name of the option (2nd field) precisely the same as the setting.name
-- 2. add a REMATCH_OPT_name_LABEL for its localized text
-- 3. create a ReflectSetting.name function if needed (optional)
rematch.optionsList = {
	{ "header", REMATCH_OPT_TARGETING_HEADER },
	{ "check", "AutoShow" },
	{ "check", "AutoLoad" },
	{ "check", "AutoMouseover", "AutoLoad" },
	{ "check", "AutoLoadAlways", "AutoLoad" },
	{ "header", REMATCH_OPT_WINDOW_HEADER },
	{ "check", "LargeWindow" },
	{ "check", "LockWindow" },
	{ "check", "LockDrawer" },
	{ "check", "ShowSidePanel" },
	{ "check", "StayForBattle" },
	{ "header", REMATCH_OPT_LOADING_HEADER },
	{ "check", "EmptyMissing" },
	{ "check", "KeepSummoned" },
	{ "check", "KeepLeveling" },
	{ "header", MISCELLANEOUS },
	{ "check", "JumpToTeam" },
	{ "check", "AutoConfirm" },
	{ "check", "BestOfSpecies" },
	{ "check", "DisableShare" }
}

rematch.ReflectSetting = {} -- table of functions to run to reflect changes for defined settings

function rematch:InitializeOptions()
	settings = RematchSettings
	local options = rematch.drawer.options
	options.header.text:SetText(REMATCH_OPTIONS)
	options.version:SetText("Rematch version "..GetAddOnMetadata("Rematch","Version"))

	local scrollFrame = rematch.drawer.options.list.scrollFrame
	scrollFrame.update = rematch.UpdateOptionsList
	scrollFrame.stepSize = 40
	HybridScrollFrame_CreateButtons(scrollFrame,"RematchOptionsListTemplate")
	local scrollBar = scrollFrame.scrollBar
	scrollBar.doNotHide = true
	scrollBar.trackBG:SetPoint("TOPLEFT",-2,15)
	scrollBar.trackBG:SetPoint("BOTTOMRIGHT",2,-15)
	scrollBar.trackBG:SetTexture("Interface\\ChatFrame\\ChatFrameBackground")
	scrollBar.trackBG:SetGradientAlpha("HORIZONTAL",.125,.125,.125,1,.05,.05,.05,1)

	-- go through each checkbutton, set its text and run its reflect if it has one
	local list = rematch.optionsList
	for i=1,#list do
		if rematch.ReflectSetting[list[i][2]] then
			rematch.ReflectSetting[list[i][2]]()
		end
	end
end

function rematch:OptionsOnShow()
	rematch.saved.header:Hide()
	rematch.current.header:Hide()
	rematch:UpdateOptionsList()
	local scrollFrame = rematch.drawer.options.list.scrollFrame
	HybridScrollFrame_SetOffset(scrollFrame,0)
	scrollFrame.scrollBar:SetValue(0)
end

function rematch:OptionsOnHide()
	rematch.saved.header:Show()
	rematch.current.header:Show()
end

function rematch:RematchOptionsButton()
	rematch.drawer.options:Show()
end

function rematch:RematchOptionsOkayButton()
	rematch.drawer.options:Hide()
end

function rematch.ReflectSetting.AutoLoad()
	rematch.ReflectSetting.AutoMouseover() -- update whether UPDATE_UNIT_MOUSEOVER should be registered
	rematch.sidePanel.autoLoad:SetChecked(settings.AutoLoad)
end

function rematch.ReflectSetting.AutoMouseover()
	if settings.AutoMouseover and settings.AutoLoad then
		rematch:RegisterEvent("UPDATE_MOUSEOVER_UNIT")
	else
		rematch:UnregisterEvent("UPDATE_MOUSEOVER_UNIT")
	end
end

function rematch.ReflectSetting.LargeWindow()
	-- instead of a straight SetScale, we'll adjust position so it stays at the same
	-- relative position on the screen, growing out/in from the bottomleft corner
	local oldScale = rematch:GetEffectiveScale()
	settings.XPos = rematch:GetLeft()
	settings.YPos = rematch:GetBottom()
	rematch:SetScale(settings.LargeWindow and 1.25 or 1.0)
	local newScale = rematch:GetEffectiveScale()
	settings.XPos = (settings.XPos*oldScale)/newScale
	settings.YPos = (settings.YPos*oldScale)/newScale
	rematch:ClearAllPoints()
	rematch:SetPoint("BOTTOMLEFT",settings.XPos,settings.YPos)
	RematchFloatingPetCard:SetScale(settings.LargeWindow and 1.2 or 1)
end

function rematch.ReflectSetting.BestOfSpecies()
	rematch:SetupBestOfSpecies()
end

function rematch:ToggleAutoLoad()
	settings.AutoLoad = not settings.AutoLoad
	if rematch.drawer.options:IsVisible() then
		rematch:UpdateOptionsList()
	else
		print(REMATCH_AUTOLOAD_IS_NOW,settings.AutoLoad and REMATCH_AUTOLOAD_ENABLED or REMATCH_AUTOLOAD_DISABLED)
	end
end

function rematch:UpdateOptionsList()
	local list = rematch.optionsList
	local numData = #list
	local scrollFrame = rematch.drawer.options.list.scrollFrame
	local offset = HybridScrollFrame_GetOffset(scrollFrame)
	local buttons = scrollFrame.buttons
	for i=1, #buttons do
		local index = i + offset
		local button = buttons[i]
		button.index = nil
		button:Hide()
		if ( index <= numData) then
			button.index = index
			local label = _G["REMATCH_OPT_"..list[index][2].."_LABEL"]
			button.name:SetText(label and label or list[index][2])
			if list[index][1]=="header" then
				button.name:SetTextColor(1,.82,0)
				button.back:Show()
				button.check:Hide()
			elseif list[index][1]=="check" then
				button.back:Hide()
				button.check:SetPoint("LEFT",list[index][3] and 16 or 6,0)
				if list[index][3] and not settings[list[index][3]] then
					button.check:SetEnabled(0) -- disable dependant option if parent unchecked
					button.name:SetTextColor(.5,.5,.5)
				else
					button.check:SetEnabled(1)
					button.name:SetTextColor(1,1,1)
				end
				button.check:SetChecked(settings[list[index][2]])
				button.check:Show()
			end
			button:Show()
		end
	end
	HybridScrollFrame_Update(scrollFrame,20*numData,20)
end

function Rematch:OptionsCheckOnEnter()
	local index = self:GetParent().index
	local list = rematch.optionsList
	if index and list[index] and list[index][2] then
		local tooltip = _G["REMATCH_OPT_"..list[index][2].."_DESC"]
		if tooltip then
			rematch.drawer.options.tooltip.text:SetText(tooltip)
		end
	end
end

function Rematch:OptionsCheckOnLeave()
	rematch.drawer.options.tooltip.text:SetText("")
end

function rematch:OptionsCheckOnClick()
	local index = self:GetParent().index
	local list = rematch.optionsList
	if index and list[index] and list[index][2] then
		local setting = list[index][2]
		settings[setting] = self:GetChecked()
		rematch:UpdateOptionsList()
		if rematch.ReflectSetting[setting] then
			rematch.ReflectSetting[setting]()
		end
	end
end