--[[ Roster

	This module handles the list of pets that will be displayed in the browser.

	The actual displaying is handled in browser.lua/xml.  This just populates a list of pets
	based on filters and flags.

	The UI's live collected/type/source filters are allowed to apply normally and "first".
	If strong vs or tough vs filter is enabled, pet type filter is cleared to let all types through.
	The UI's search filter is not allowed to be applied (search is cleared when pets gathered).
	Our own search filter looks for text from what's left above (abilities too).
	Then a post-search filter discards types for strong vs and tough vs if we're in the mode.
	Some breed caching is also done here if addon Battle Pet Breed ID enabled.

		GetNumPets() -- returns total number of pets
		GetPetByIndex(index) -- returns petID of owned pet or speciesID of missing pet
		Update() -- updates roster with internal PET_JOURNAL_LIST_UPDATE force flag

]]

local rematch = Rematch
rematch.roster = {}
local roster = rematch.roster

roster.toughTable = {8,4,2,9,1,10,5,3,6,7} -- types that indexed types are tough vs
roster.strongTable = {4,1,6,5,8,2,9,10,3,7} -- types that indexed types are strong vs

roster.pets = {} -- table of pet indexes, created in GenerateIndexes, owned or missing, regardless of filters
roster.breeds = {} -- BattlePetBreedID creates insane garbage churn, cache a table of pet breeds
roster.abilityList = rematch.abilityList -- reusable table for C_PetJournal.GetAbilityList
roster.levelList = rematch.levelList -- passed to C_PetJournal.GetAbilityList (prevents a ton of garbage creation)
roster.typeFilter = {nil,{},{}} -- 1=type, 2=strong, 3=tough
roster.rarityFilter = {} -- 1-4 poor-rare filter
roster.currentZoneFilter = nil -- whether to filter to pets in the current zone
roster.favoriteFilter = nil -- whether to filter favorites (not going to use default)

-- returns number of pets in roster
function roster:GetNumPets()
	return #roster.pets
end

-- returns a petID/speciesID by roster index
function roster:GetPetByIndex(index)
	return roster.pets[index]
end

function roster:GetPetNameByIndex(index)
	local customName,realName,_
	local petID = roster.pets[index]
	if type(petID)=="string" then
		_,customName,_,_,_,_,_,realName = C_PetJournal.GetPetInfoByPetID(roster.pets[index])
	elseif type(petID)=="number" then
		realName = C_PetJournal.GetPetInfoBySpeciesID(petID)
	end
	return customName or realName
end

-- this populates roster.pets with petIDs or speciesIDs depending on current filters
-- note it does not (and should never) do anything to trigger a PET_JOURNAL_LIST_UPDATE
function roster:Update()
	wipe(roster.pets)

	local checkStrong = not roster:IsTypeFilterClear(2)
	local checkTough = not roster:IsTypeFilterClear(3)
	-- gather pets
	for i=1,C_PetJournal.GetNumPets() do
		local candidate = roster:FilterPetByIndex(i,checkStrong,checkTough,not roster:IsRarityFilterClear(),roster:GetCurrentZoneFilter() and GetRealZoneText())
		if candidate then
			tinsert(roster.pets,candidate)
		end
	end

end




--[[ Search box ]]

function roster:SetSearch(text)
	if roster.searchText==text then
		return
	end

	roster.searchText = nil
	roster.searchMinLevel = nil
	roster.searchMaxLevel = nil

	if not text or text=="" or text==SEARCH then
		-- keep everything nil
	else
		local minLevel,maxLevel = roster:TestSearchForLevelRange(text)
		if minLevel then
			roster.searchMinLevel = minLevel
			roster.searchMaxLevel = maxLevel
		else
			roster.searchText = text
			-- searchMask is a literal/case-insensitive pattern ("Xu-Fu" becomes "[xX][uU]%-[fF][uU]")
			roster.searchMask = roster.searchText:gsub("[%(%)%.%%%+%-%*%?%[%]%^%$]",function(c) return "%"..c end):gsub("%a",function(c) return format("[%s%s]",c:lower(),c:upper()) end)
		end
	end
end

-- returns min,max levels if one can be determined from passed text
-- <level <=level >level >=level =level level-level are valid returns
function roster:TestSearchForLevelRange(text)
	local minLevel,maxLevel
	maxLevel = text:match("^<(%d+)")
	if maxLevel then return 0,tonumber(maxLevel)-1 end
	minLevel = text:match("^>(%d+)")
	if minLevel then return tonumber(minLevel)+1,25 end
	minLevel = text:match("^=(%d+)")
	if minLevel then return tonumber(minLevel),tonumber(minLevel) end
	minLevel,maxLevel = text:match("^(%d+)%-(%d+)")
	if minLevel then return tonumber(minLevel),tonumber(maxLevel) end -- "level-level"
	maxLevel = text:match("^<=(%d+)")
	if maxLevel then return 0,tonumber(maxLevel) end
	minLevel = text:match("^>=(%d+)")
	if minLevel then return tonumber(minLevel),25 end
end


--[[ Type Filter ]]

-- this function takes mode (1/nil=type, 2=strong, 3=tough) and sets that type to value.
-- it handles the brain-hurting reverse flags for normal types as well as clearing
-- the type when all types are enabled
function roster:SetTypeFilter(mode,index,value)
	if value==false then
		value = nil -- we use next(table) to quickly tell if a strong/tough filter is clear
	end
	if roster.typeFilter[mode] then
		-- if mode 2 or 3, just set its value
		roster.typeFilter[mode][index] = value
	else
		-- if mode 1 and all types aren't enabled, then set the individual type
		if not roster:IsTypeFilterClear() then
			C_PetJournal.SetPetTypeFilter(index,value)
		else -- if all types enabled, turn off all but the one we're enabling
			for i=1,10 do
				C_PetJournal.SetPetTypeFilter(i,i==index)
			end
		end
	end
	-- now go through and see if we just turned off last type for normal type, or
	-- turned on last type for strong/tough
	local missing
	for i=1,10 do
		if roster.typeFilter[mode] then
			missing = missing or not roster:GetTypeFilter(mode,i)
		else
			missing = missing or roster:GetTypeFilter(1,i)
		end
	end
	-- everything is now enabled, clear the filter
	if not missing then
		roster:ClearTypeFilter(mode)
	end
end

function roster:GetTypeFilter(mode,index)
	if roster.typeFilter[mode] then
		return roster.typeFilter[mode][index]
	else
		return not C_PetJournal.IsPetTypeFiltered(index) -- note the not!
	end
end

function roster:IsTypeFilterClear(mode)
	if roster.typeFilter[mode] then
		return not next(roster.typeFilter[mode])
	else
		for i=1,10 do
			if C_PetJournal.IsPetTypeFiltered(i) then
				return false
			end
		end
		return true
	end
end

-- to prevent triggering an unnecessary PET_JOURNAL_LIST_UPDATE, add all pet types only if needed
function roster:ClearTypeFilter(mode)
	if roster.typeFilter[mode] then
		wipe(roster.typeFilter[mode])
	else
		for i=1,C_PetJournal.GetNumPetTypes() do
		  if C_PetJournal.IsPetTypeFiltered(i) then -- IsPetTypeFiltered returns true if it's unchecked
				C_PetJournal.AddAllPetTypesFilter() -- if anything unchecked, check them all
				break
		  end
		end
	end
end

--[[ Rarity Filter ]]

function roster:SetRarityFilter(rarity,value)
	if not value then value = nil end
	roster.rarityFilter[rarity] = value
end

function roster:GetRarityFilter(rarity)
	return roster.rarityFilter[rarity]
end

function roster:IsRarityFilterClear()
	return not next(roster.rarityFilter)
end

function roster:ClearRarityFilter()
	wipe(roster.rarityFilter)
end

--[[ Current Zone Filter ]]

function roster:SetCurrentZoneFilter(value)
	roster.currentZoneFilter = value
end

function roster:GetCurrentZoneFilter()
	return roster.currentZoneFilter
end

--[[ Favorite Filter ]]

function roster:SetFavoriteFilter(value)
	roster.favoriteFilter = value
end

function roster:GetFavoriteFilter()
	return roster.favoriteFilter
end

--[[ Filtering ]]

-- returns true if the speciesID has a petType we want to show
-- note that roster.abilityList should be populated before calling this
function roster:FilterPetType(speciesID,petType,mode)

	if mode==3 then
		for typeIndex in pairs(roster.typeFilter[3]) do
			if roster.toughTable[typeIndex] == petType then
				return true
			end
		end
	elseif mode==2 then
		for i=1,#roster.abilityList do
			local abilityType, noHints = select(7,C_PetBattles.GetAbilityInfoByID(roster.abilityList[i]))
			if not noHints then
				for typeIndex in pairs(roster.typeFilter[2]) do
					if roster.strongTable[typeIndex] == abilityType then
						return true
					end
				end
			end
		end
	else -- normal type
		return true -- we use all filtered pets for normal
	end

	return false
end

-- returns true if one of the passed parameters has a hit with the searchMask
-- it will also search abilities populated in roster.abilityList
function roster:FilterSearchText(...)

	for i=1,select("#",...) do
		local text = select(i,...)
		if text and text:match(roster.searchMask) then
			return true
		end
	end

	for i=1,#roster.abilityList do
		local _,name,_,_,description = C_PetBattles.GetAbilityInfoByID(roster.abilityList[i])
		if name:match(roster.searchMask) or description:match(roster.searchMask) then
			return true
		end
	end

end

-- returns the petID or speciesID if pet journal index should be listed in browser
-- if checkType is true, a test for type is also applied
-- if searchText is true, a test for searchMask is also applied
function roster:FilterPetByIndex(index,checkStrong,checkTough,checkRarity,currentZone)

	local	petID, speciesID, owned, customName, level, favorite, _, speciesName, _, petType, _, source, description, isWild, canBattle, isTradeable, _, obtainable = C_PetJournal.GetPetInfoByIndex(index)
	C_PetJournal.GetPetAbilityList(speciesID, roster.abilityList, roster.levelList)

	if not canBattle and (checkStrong or checkTough) then
		return false -- toss out non-battle pets for strong and tough searches
	end

	if roster:GetFavoriteFilter() and not favorite then
		return false
	end

	if currentZone then
		if not source or not source:match(currentZone) then
			return false
		end
	end

	-- check rarity if flag set
	if checkRarity then
		if petID then
			local _, maxHealth, power, speed, rarity = C_PetJournal.GetPetStats(petID)
			if not roster:GetRarityFilter(rarity) then
				return false
			end
		else
			return false
		end
	end

	-- check tough types first since it's fastest check
	if checkTough then
		if not roster:FilterPetType(speciesID,petType,3) then
			return false
		end
	end
	if checkStrong then
		if not roster:FilterPetType(speciesID,petType,2) then
			return false
		end
	end

	-- if there's a min,max level (search string is <25 or =25 or 11-17 or >5 etc)
	if roster.searchMinLevel then
		if level<roster.searchMinLevel or level>roster.searchMaxLevel then
			return false
		end
	end

	-- if there's a legitimate text search string
	if roster.searchText then
		if not roster:FilterSearchText(customName,speciesName,source) then
			return false
		end
	end

	return petID or speciesID,favorite
end

--[[ Sorting ]]


--[[ filterResults ]]

function roster:GetFilterResults()
	local filters = REMATCH_FILTERS_STUB

	if roster.searchText then
		filters = filters..REMATCH_FILTERS_SEARCH
	end
	if roster.searchMinLevel then
		filters = filters..REMATCH_FILTERS_LEVEL
	end
	if not roster:IsTypeFilterClear() then
		filters = filters..REMATCH_FILTERS_TYPE
	end
	if not roster:IsTypeFilterClear(2) then
		filters = filters..REMATCH_FILTERS_STRONG
	end
	if not roster:IsTypeFilterClear(3) then
		filters = filters..REMATCH_FILTERS_TOUGH
	end
	for i=1,C_PetJournal.GetNumPetSources() do
		if C_PetJournal.IsPetSourceFiltered(i) then
			filters = filters..REMATCH_FILTERS_SOURCES
			break
		end
	end
	if not roster:IsRarityFilterClear() then
		filters = filters..REMATCH_FILTERS_RARITY
	end
	if C_PetJournal.IsFlagFiltered(LE_PET_JOURNAL_FLAG_COLLECTED) or C_PetJournal.IsFlagFiltered(LE_PET_JOURNAL_FLAG_NOT_COLLECTED) then
		filters = filters..REMATCH_FILTERS_COLLECTED
	end
	if roster:GetFavoriteFilter() then
		filters = filters..REMATCH_FILTERS_FAVORITES
	end
	if roster:GetCurrentZoneFilter() then
		filters = filters..REMATCH_FILTERS_ZONE
	end

	filters = filters:gsub(", $","")

	return (filters~=REMATCH_FILTERS_STUB) and filters
end

function roster:ClearAllFilters()
	rematch:ClearAllTypeFilters()
	roster:ClearRarityFilter()
	roster:SetSearch("")
	roster:SetCurrentZoneFilter(nil)
	roster:SetFavoriteFilter(nil)
	C_PetJournal.AddAllPetSourcesFilter()
	C_PetJournal.SetFlagFilter(LE_PET_JOURNAL_FLAG_COLLECTED,true)
	C_PetJournal.SetFlagFilter(LE_PET_JOURNAL_FLAG_NOT_COLLECTED,true)
	C_PetJournal.SetFlagFilter(LE_PET_JOURNAL_FLAG_FAVORITES,false)
end
